@@ -38,7 +38,6 @@ typedef struct {
 	GMarkupParseContext *context;
 
 	const gchar         *path;
-	gchar               *base;
 
 	/* Top node of book */
 	GNode               *book_node;
@@ -56,6 +55,18 @@ typedef struct {
 	gint                 version;
 } DhParser;
 
+static void
+dh_parser_free (DhParser *parser)
+{
+        // NOTE: priv->book_tree and priv->keywords do not need to be freed
+        // because they're only used to store the locations for the return
+        // params of dh_parser_read_file()
+
+        g_markup_parse_context_free (parser->context);
+        g_free (parser->m_parser);
+        g_free (parser);
+}
+
 static void
 parser_start_node_book (DhParser             *parser,
                         GMarkupParseContext  *context,
@@ -67,7 +78,7 @@ parser_start_node_book (DhParser             *parser,
         gint         i, j;
         gint         line, col;
         gchar       *title = NULL;
-        const gchar *base = NULL;
+        gchar *base = NULL;
         const gchar *name = NULL;
         const gchar *uri = NULL;
         const gchar *lang = NULL;
@@ -78,7 +89,7 @@ parser_start_node_book (DhParser             *parser,
                 g_set_error (error,
                              DH_ERROR,
                              DH_ERROR_MALFORMED_BOOK,
-                             _("Expected '%s' got '%s' at line %d, column %d"),
+                             _("Expected '%s', got '%s' at line %d, column %d"),
                              "book", node_name, line, col);
                 return;
         }
@@ -111,7 +122,7 @@ parser_start_node_book (DhParser             *parser,
                         }
                 }
                 else if (g_ascii_strcasecmp (attribute_names[i], "base") == 0) {
-                        base = attribute_values[i];
+                        base = g_strdup (attribute_values[i]);
 			}
                 else if (g_ascii_strcasecmp (attribute_names[i], "link") == 0) {
                         uri = attribute_values[i];
@@ -126,33 +137,33 @@ parser_start_node_book (DhParser             *parser,
                 g_set_error (error,
                              DH_ERROR,
                              DH_ERROR_MALFORMED_BOOK,
-                             _("\"title\", \"name\", and \"link\" elements are "
+                             _("\"title\", \"name\" and \"link\" elements are "
                                "required at line %d, column %d"),
                              line, col);
                 g_free (title);
                 return;
         }
 
-        if (base) {
-                parser->base = g_strdup (base);
-        } else {
-                parser->base = g_path_get_dirname (parser->path);
+        if (!base) {
+                base = g_path_get_dirname (parser->path);
         }
 
         link = dh_link_new (DH_LINK_TYPE_BOOK,
-                            parser->base,
+                            base,
                             name,
                             title,
                             NULL,
                             NULL,
                             uri);
+        g_free (base);
 
-        *parser->keywords = g_list_prepend (*parser->keywords, link);
+        *parser->keywords = g_list_prepend (*parser->keywords, dh_link_ref (link));
 
-        parser->book_node = g_node_new (link);
+        parser->book_node = g_node_new (dh_link_ref (link));
         g_node_prepend (parser->book_tree, parser->book_node);
         parser->parent = parser->book_node;
         g_free (title);
+        dh_link_unref (link);
 }
 
 static void
@@ -175,7 +186,7 @@ parser_start_node_chapter (DhParser             *parser,
                 g_set_error (error,
                              DH_ERROR,
                              DH_ERROR_MALFORMED_BOOK,
-                             _("Expected '%s' got '%s' at line %d, column %d"),
+                             _("Expected '%s', got '%s' at line %d, column %d"),
                              "sub", node_name, line, col);
                 return;
         }
@@ -239,7 +250,7 @@ parser_start_node_keyword (DhParser             *parser,
                 g_set_error (error,
                              DH_ERROR,
                              DH_ERROR_MALFORMED_BOOK,
-                             _("Expected '%s' got '%s' at line %d, column %d"),
+                             _("Expected '%s', got '%s' at line %d, column %d"),
                              "keyword", node_name, line, col);
                 return;
         }
@@ -249,7 +260,7 @@ parser_start_node_keyword (DhParser             *parser,
                 g_set_error (error,
                              DH_ERROR,
                              DH_ERROR_MALFORMED_BOOK,
-                             _("Expected '%s' got '%s' at line %d, column %d"),
+                             _("Expected '%s', got '%s' at line %d, column %d"),
                              "function", node_name, line, col);
                 return;
         }
@@ -602,9 +613,7 @@ dh_parser_read_file (const gchar  *path,
 	if (io) {
                 g_io_channel_unref (io);
         }
-	g_markup_parse_context_free (parser->context);
-	g_free (parser->m_parser);
-	g_free (parser);
+	dh_parser_free (parser);
 
 	return result;
 }
